@@ -2,7 +2,7 @@
  *  mod_musicindex.h
  *  mod_musicindex
  *
- *  $Id: mod_musicindex.h 670 2006-07-31 21:34:34Z varenet $
+ *  $Id: mod_musicindex.h 678 2006-10-14 19:49:51Z varenet $
  *
  *  Created by Thibaut VARENE on Thu Mar 20 2003.
  *  Copyright (c) 2003-2006 Regis BOUDIN
@@ -201,6 +201,22 @@
 
 #else
 
+ #define ap_base64encode apr_base64_encode
+ #define ap_table_get apr_table_get
+ #define ap_palloc apr_palloc
+ #define ap_destroy_pool apr_pool_destroy
+ #define ap_table_setn apr_table_setn
+ #define ap_pstrcat apr_pstrcat
+ #define ap_base64encode_len apr_base64_encode_len
+ #define ap_cpystrn apr_cpystrn
+ #define ap_pcalloc apr_pcalloc
+ #define ap_pstrdup apr_pstrdup
+ #define ap_psprintf apr_psprintf
+ #define ap_pstrndup apr_pstrndup
+ #define ap_base64decode_len apr_base64_decode_len
+ #define ap_http_method ap_http_scheme
+ #define ap_base64decode apr_base64_decode
+
  #include <apr_strings.h>
  #include <apr_base64.h>
  #include <ap_compat.h>
