@@ -2,11 +2,11 @@
  *  playlist.c
  *  mod_musicindex
  *
- *  $Id: playlist.c 662 2006-07-09 21:05:07Z varenet $
+ *  $Id: playlist.c 670 2006-07-31 21:34:34Z varenet $
  *
  *  Created by Thibaut VARENE on Thu Mar 20 2003.
  *  Copyright (c) 2003-2004 Regis BOUDIN
- *  Copyright (c) 2003-2005 Thibaut VARENE
+ *  Copyright (c) 2003-2006 Thibaut VARENE
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License version 2.1,
@@ -20,7 +20,7 @@
  *
  * @author Regis Boudin
  * @author Thibaut Varene
- * @version $Revision: 662 $
+ * @version $Revision: 670 $
  * @date 2003-2006
  *
  * This is where the magic takes place. Here are the functions in charge of
@@ -448,10 +448,8 @@ mu_ent *make_music_entry(request_rec *r, apr_pool_t *pool,
 		p->flags |= EF_ALLOWTARBALL;	
 	if (soptions & MI_ALLOWRSS)
 		p->flags |= EF_ALLOWRSS;
-
 	if ((soptions & MI_CUSTOM) == 0)
-		p->file += strlen(r->parsed_uri.path);	/* remove the path before the filename except for custom playlists */
-
+		p->file += strlen(r->parsed_uri.path);  /* remove the path before the filename except for custom playlists */
 	if (conf->options & MI_TARBALL)
 		p->filename = ap_pstrdup(pool, names->filename);
 
@@ -514,12 +512,12 @@ mu_ent *make_music_entry(request_rec *r, apr_pool_t *pool,
  *
  * This function generates a list containing all the files referenced in the
  * conf->custom_list string. It handles parameters in a way so that it can
- * handle indifferently strings for cookies or playlists.
+ * handle indifferently strings for cookies or on-the-fly playlists.
  *
  * @param r Apache request_rec struct
  * @param conf MusicIndex configuration paramaters struct
  *
- * @return 0 on success, error code otherwise.
+ * @return a list of mu_ent on success, NULL otherwise.
  */
 mu_ent *build_custom_list(request_rec *r, const mu_config *const conf)
 {
@@ -527,33 +525,40 @@ mu_ent *build_custom_list(request_rec *r, const mu_config *const conf)
 	mu_ent_names	names;
 	const char	*args;
 	mu_ent		*mobile_ent = NULL, *custom = NULL, *result = NULL;
-	register unsigned short	i;
-	char		*p;
+	char		*p, *decodeduri;
+	short		direct = 0;
 
 	if (conf->custom_list == NULL)
 		return NULL;
 
 	args = conf->custom_list;
 
+	
 	if (strncmp(args, "playlist=", 9) == 0)
-		args += 9;
+		args += 9;	/* cookie */
+	else if (strncmp(args, "file=", 5) == 0)
+		direct = 1;	/* direct request (Play Selected) */
 
 	while ((*args != '\0') && (*args != ';')) {
 		int local_options;
 		p = ap_getword(r->pool, &args, '&');
-
-		if (strncmp("file=", p, 5) == 0)
-			p += 5;
-		else if (strchr(p, '='))
+		if (direct && (!strncmp(p, "file=", 5))) {
+			p+=5;
+			ap_unescape_url(p);
+		}
+		else if (direct)
 			continue;
 
-		for (i=0; p[i]; i++) {
-			if (p[i] == '+')
-				p[i] = ' ';
-		}
-
-		ap_unescape_url(p);
-
+		decodeduri = (char *)ap_palloc(r->pool, 1 + ap_base64decode_len(p));
+		if (!decodeduri)
+			return NULL;
+		
+		ap_base64decode(decodeduri, p);
+		/* decodeduri is unescaped. We need to escape it before
+		   passing a new request to the server */
+		p = ap_escape_uri(r->pool, decodeduri);
+		
+		/* we do that to avoid nasty accesses if somebody crafts a uri */
 		subreq = ap_sub_req_lookup_uri(p, r, NULL);
 
 		if (subreq == NULL)
@@ -566,10 +571,12 @@ mu_ent *build_custom_list(request_rec *r, const mu_config *const conf)
 
 		ap_destroy_sub_req(subreq);
 		
+		/* adds entries bottom-up */
 		custom = make_music_entry(r, r->pool, custom, conf, &names,
 			(MI_CUSTOM | (local_options & MI_ALLOWFETCH)));
 	}
 
+	/* reorder the list top-down */
 	for (result = NULL; custom; custom = mobile_ent) {
 		mobile_ent = custom->next;
 		custom->next = result;
